/*
	Store 16-bit opmode value in STM32F0 Flash
	gbm 11'2015
*/
#include "board.h"

#if defined (STM32F091)
#define DFLASH_PGSIZE	2048
#elif defined (STM32F051)
#define DFLASH_PGSIZE	1024
#endif
#define DFLASH_PGENTRIES	(DFLASH_PGSIZE / 2)
#define DFLASH_ADDR	0x08003000
#define dflash16	((uint16_t *)DFLASH_ADDR)
#define dflash_pg0	((uint16_t *)DFLASH_ADDR)
#define dflash_pg1	((uint16_t *)(DFLASH_ADDR + DFLASH_PGSIZE))

static uint16_t idx = 0;	// flash data index

uint16_t flash_getmode(void)
{
	uint16_t val = 0;
	
	if (dflash_pg0[0] != 0xffff && dflash_pg1[0] != 0xffff)
	{
		// both pages in use - one should be erased
		FLASH->CR = FLASH_CR_PER;
		FLASH->SR = FLASH_SR_EOP;
		// select page to erase
		FLASH->AR = dflash_pg0[DFLASH_PGENTRIES - 1] < dflash_pg1[DFLASH_PGENTRIES - 1] ?
			(uint32_t)dflash_pg0 : (uint32_t)dflash_pg1;
		FLASH->CR = FLASH_CR_PER | FLASH_CR_STRT;
	}
	if (dflash_pg1[0] != 0xffff)
		idx = DFLASH_PGENTRIES;	// use page 1
	while (idx < DFLASH_PGENTRIES * 2 && dflash16[idx] != 0xffff)
		val = dflash16[idx ++];
	if (idx == DFLASH_PGENTRIES * 2)
		idx = 0;
	return val;
}
void flash_storemode(uint16_t m)
{
	FLASH->CR = FLASH_CR_PG;
	FLASH->SR = FLASH_SR_EOP;
	dflash16[idx] = m;
}
