/*
	Store 16-bit opmode value in STM32F4 Flash
	gbm 11'2015
*/
#include "board.h"

#define DFLASH_ADDR	0x08008000	// 2 pages starting from
#define DFLASH_PGSIZE	16384
#define DFLASH_PGENTRIES	4096
#define dflash	((uint32_t *)DFLASH_ADDR)
#define dflash_pg0	((uint32_t *)DFLASH_ADDR)
#define dflash_pg1	((uint32_t *)(DFLASH_ADDR + DFLASH_PGSIZE))

static uint16_t idx = 0;	// flash data index

uint16_t flash_getmode(void)
{
	uint16_t val = 0;
	
	if (dflash_pg0[0] != 0xffffffff && dflash_pg1[0] != 0xffffffff)
	{
		// both pages in use - one should be erased
		FLASH->SR = FLASH_SR_EOP;
		// select page to erase
		FLASH->CR = FLASH_CR_PSIZE32 | FLASH_CR_SER
			| FLASH_CR_SNBV(dflash_pg0[DFLASH_PGENTRIES - 1] < dflash_pg1[DFLASH_PGENTRIES - 1] ? 2 : 3);
		FLASH->CR |= FLASH_CR_STRT;
	}
	if (dflash_pg1[0] != 0xffffffff)
		idx = DFLASH_PGENTRIES;	// use page 1
	while (idx < DFLASH_PGENTRIES * 2 && dflash[idx] != 0xffffffff)
		val = dflash[idx ++];
	if (idx == DFLASH_PGENTRIES * 2)
		idx = 0;
	return val;
}
void flash_storemode(uint16_t m)
{
	FLASH->SR = FLASH_SR_EOP;
	FLASH->CR = FLASH_CR_PSIZE32 | FLASH_CR_PG;
	dflash[idx] = 0xffff0000 | m;
}
